import java.awt.geom.Path2D;
import java.awt.geom.AffineTransform;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.*;


/**
 * Clase objeto grafico de elastico.
 * @author Luis Fuentes / Benjamin Ginouves
 */
public class GRubberBand extends RubberBand implements Draggable {
   private static final int Points = 50;
   private static final Path2D.Double polyline =
                      new Path2D.Double(Path2D.WIND_EVEN_ODD,Points);
   private Path2D.Double shape;
   private Color color = Color.BLACK;
   private Vector2D graphicsPosA =null;  // to draw a spring we need
   private Vector2D graphicsPosB =null;  // extremes well definied
                                      // when it is not connected

   static {  // static initialization block
      double aux;
      polyline.moveTo (0, 0);
      for (int index = 0; index < Points;index++){
         aux =  (double)index/Points;
         polyline.lineTo(aux, 0.001*Math.sin(index*Math.PI/4));
      }
   }
   /**
    * Metodo de inicializacion de los valores del objeto.
    * @param restLength largo del elastico.
    * @param stiffness coeficiente de elasticidad del elastico.
    */
   public GRubberBand(float restLength, float stiffness) {
      super(restLength, stiffness);
      graphicsPosA = new Vector2D(0,restLength);
      graphicsPosB = new Vector2D(restLength,restLength);
      AffineTransform at = AffineTransform.getTranslateInstance(0,0);
      Vector2D v = getVector();
      at.rotate(v.getX(), v.getY());
      at.scale(v.module(), restLength*restLength/v.module());
      shape = (Path2D.Double) at.createTransformedShape(polyline);
   }
   /**
    * Metodo para especificar el elemento grafico del elastico, y posicionarlo en el frame.
    * @param g elemento grafico del bloque.
    */
   public void draw (Graphics2D g){
      Vector2D a=getPositionA();
      double ax = a.getX();
      double ay = a.getY();
      Vector2D v = getVector();
      AffineTransform at = AffineTransform.getTranslateInstance(ax, ay);
      at.rotate(v.getX(), v.getY());
      at.scale(v.module(), restLength*restLength/v.module());
      shape = (Path2D.Double) at.createTransformedShape(polyline);
      g.setColor(color);
      g.draw(shape);
   }
   /**
    * Metodo para evaluar si el punto esta contenido en el objeto grafico del elastico.
    * @param x coodenada x del extremo del elastico a evaluar.
    * @param y coordenada y del extremo del elastico a evaluar.
    * @return retorno si esta o no contenido.
    */
   public boolean contains(double x, double y){
       int p = 5*Points;
       for(int j=-p;j<p;j=j+5)
            for(int k=-p;k<p;k=k+5)
                if (shape.contains(x + j/Points,y + k/Points)) return true;
       return shape.contains(x,y);
   }
   /**
    * Metodo para mover el extremo del objeto grafico del elastico mas proximo al punto x,y.
    * @param x coordenada x a la cual se quiere mover el objeto grafico del elastico.
    * @param y coordenada y a la cual se quiere mover el objeto grafico del elastico.
    */
   public void drag(double x, double y){
      Vector2D a=getPositionA();
      Vector2D b=getPositionB();
      double da = Math.abs(a.getX()-x)+Math.abs(a.getY()-y); // distance to a
      double db = Math.abs(b.getX()-x)+Math.abs(b.getY()-y); // distance to a
      if (da < db) {
         detachBlock_a();
         graphicsPosA.moveTo(x,y);
      }
      else {
         detachBlock_b();
         graphicsPosB.moveTo(x,y);
      }
   }
   /**
    * Metodo para obtener la posicion del extremo A del elastico.
    * @return vectro de posicion del extremo A del elastico.
    */
   public Vector2D getPositionA() {
      Vector2D p = super.getPositionA();
      if (p != null)
         return p;
      else
         return graphicsPosA;
   }
   /**
    * Metodo para obtener la posicion del extremo B del elastico.
    * @return vector de posicion del extremo B del elastico.
    */
   public Vector2D getPositionB() {
      Vector2D p = super.getPositionB();
      if (p != null)
         return p;
      else
         return graphicsPosB;
   }
   /**
    * Metodo para actualizar el estado y posicion del objeto grafico del elastico.
    */
   public void updateState(){
      if (block_a==null && block_b==null) return;
      if (block_a!=null) {
         Vector2D v = getVector().unitary().times(restLength);
         graphicsPosB = getPositionA().plus(v);
         return;
      }
      if (block_b!=null) {
         Vector2D v = getVector().unitary().times(restLength);
         graphicsPosA = getPositionB().minus(v);
         return;
      }
   }
   /**
    * Metodo para enlazar el extremo A o B del elastico al bloque.
    * @param x coordenada del extremo del elastico.
    * @param y coordenada del extremo del elastico.
    * @param bsc arreglos de los objetos del panel.
    */
   public void attachNearElement(double x, double y, BlockSpringConfiguration bsc){
      Vector2D a=getPositionA();
      if ((a.getX()==x) && (a.getY()==y)) {
         GBlock block = bsc.findBlock(x,y);
         if (block!=null) attachBlock_a(block);
         else
         return;
      }
      Vector2D b=getPositionB();
      if ((b.getX()==x) && (b.getY()==y))  {
         GBlock block = bsc.findBlock(x,y);
         if (block!=null) attachBlock_b(block);
         return;
      }
   }
   /**
    * Metodo para cambiar el color del objeto a rojo.
    */
   public void setSelectedColor() {
          color = Color.RED;
   }
   /**
    * Metodo para cambiar el color del objeto a gris.
    */
   public void setDefaultColor() {
      color = Color.BLACK;
   }
}